#include <GOSScript\GOSScriptHeaders.hpp>
#include "SampleClass.hpp"

mySample::mySample(int px, int py, int pcolor)
{
	x = px;
	y = py;
	color = pcolor;
	gosScript_RegisterVariable("x",&x,GOSVAR_INT,0,this);
	gosScript_RegisterVariable("y",&y,GOSVAR_INT,0,this);
	gosScript_RegisterVariable("color",&color,GOSVAR_INT,0,this);
	myScript = gosScript_ExecuteScript("assets\\scripts\\mySample.script",0x1000,this);
}

mySample::~mySample()
{
	gosScript_UnregisterAll(this);
	gosScript_EndScript(myScript);
}